# substitute send_mail function.
from django.conf import settings
from django.contrib.auth.models import User
from django.core import mail
from .mail import send_mail as send_html_mail

mail.send_mail = send_html_mail

# https://docs.djangoproject.com/en/dev/topics/auth/customizing/#referencing-the-user-model
try:
    from django.contrib.auth import get_user_model
    User = get_user_model()
except ImportError:
    pass

def email_user(self, subject, message, from_email=None):
     """
     Sends an email to this User.
     """
     send_html_mail(subject, message, from_email, [self.email])

User.email_user = email_user

if 'mailer' in settings.INSTALLED_APPS:
    import mailer
    mailer.send_mail = send_html_mail


from django import template
from subprocess import Popen, PIPE
from bs4 import BeautifulSoup

register = template.Library()

@register.filter
def html2text(value):
    """
    Pipes given HTML string into the text browser W3M, which renders it.
    Rendered text is grabbed from STDOUT and returned.
    """
    try:
        cmd = "w3m -dump -T text/html -O utf-8"
        proc = Popen(cmd, shell = True, stdin = PIPE, stdout = PIPE)
        return proc.communicate(str(value))[0]
    except OSError:
        # something bad happened, so just return the input
        return value

@register.filter
def extract_urllinks(value, template='%(text)s (%(url)s)'):
    '''
    Extract urls from links and put it to brackets after links. Useful for generating plain version of email body from html
    '''
    html = BeautifulSoup(value)
    for link in html.findAll('a'):
        text = ''.join(map(unicode, link.contents)).strip()
        if link.get('href') and link.get('href') != text:
            result = template % {
                'text': text,
                'url': link['href'].replace('\n',''),
            }
        elif text:
            result = text

        link.replaceWith(result)
    return html


from django.conf import settings
from django.core.mail import EmailMultiAlternatives, EmailMessage
from .templatetags.email_html import html2text, extract_urllinks
import re

def send_mail(subject, message, from_email=None, recipient_list=None,
              fail_silently=False, auth_user=None, auth_password=None,
              connection=None):
    '''
    Replacement for monkey-patching Django's send_mail function for sending html email by default
    '''
    if recipient_list is None:
        raise ValueError('You must specified recipient_list attribute')

    bcc_addrs = list(getattr(settings, 'EMAIL_BCC_ADDRESSES', []))
    admins = [a[1] for a in settings.ADMINS] if getattr(settings, 'EMAIL_ADMIN_DUPLICATE', False) else []
    bcc_addrs.extend(admins)
    from_email = from_email or settings.DEFAULT_FROM_EMAIL
    subject = settings.EMAIL_SUBJECT_PREFIX + subject.replace('\n', '')

    if message.find('<html') != -1:
        message_plaintext = html2text(extract_urllinks(message))
        message_plaintext = re.sub(r'http://\n', 'http://', message_plaintext)
        if 'mailer' in settings.INSTALLED_APPS:
            from mailer import send_html_mail
            return send_html_mail(subject=subject, message=message_plaintext, message_html=message,
                                   from_email=from_email, recipient_list=recipient_list, bcc=bcc_addrs,
                                   fail_silently=fail_silently, auth_user=auth_user,
                                   auth_password=auth_password)
        else:
            email = EmailMultiAlternatives(subject=subject, body=message_plaintext,
                                           from_email=from_email, to=recipient_list,
                                           bcc=bcc_addrs, connection=connection)
            email.attach_alternative(message, "text/html")
            return email.send(fail_silently=fail_silently)

    else:
        email = EmailMessage(subject, message, from_email, recipient_list, bcc_addrs)
        email.send(fail_silently=fail_silently)
